\NeedsTeXFormat{LaTeX2e}[2020/02/02]
\ProvidesClass{tudscr-vpa}[2020/05/05 v1.02 style template for TUD-Script (VPA)]
\RequirePackage{tudscrbase}
\DefineFamily{vpa}
\DefineFamilyMember{vpa}
\newcommand*\vpa@baseclass{}
\let\vpa@baseclass\relax
\AtEndOfClass{\RelaxFamilyKey{vpa}{baseclass}}
\DefineFamilyKey{vpa}{baseclass}{%
  \FamilySetNumerical{vpa}{baseclass}{@tempa}{%
    {book}{0},{scrbook}{0},{tudscrbook}{0},%
    {report}{1},{scrreprt}{1},{scrreport}{1},{tudscrreprt}{1},{tudscrreport}{1}%
  }{#1}%
  \ifx\FamilyKeyState\FamilyKeyStateProcessed%
    \ifcase\@tempa\relax%
      \def\vpa@baseclass{tudscrbook}%
    \else%
      \def\vpa@baseclass{tudscrreprt}%
    \fi%
  \fi%
}
\AtEndOfClass{\RelaxFamilyKey{vpa}{pdfminorversion}}
\DefineFamilyKey{vpa}{pdfminorversion}{%
  \FamilyKeyStateUnknownValue%
  \Ifisinteger{#1}{%
    \ifnum\numexpr#1\relax>\z@%
      \iftutex%
        \ifluatex%
          \directlua{pdf.setminorversion(#1)}%
          \FamilyKeyStateProcessed%
        \fi%
        \ifxetex%
          \special{pdf:minorversion #1}%
          \FamilyKeyStateProcessed%
        \fi%
      \else%
        \ifpdftex%
          \pdfminorversion=#1%
          \FamilyKeyStateProcessed%
        \fi%
      \fi%
    \fi%
  }{}%
}
\newcount\vpa@type
\vpa@type=\m@ne
\newif\if@vpa@student
\AtEndOfClass{\RelaxFamilyKey{vpa}{vpatype}}
\DefineFamilyKey{vpa}{vpatype}{%
  \FamilySetNumerical{vpa}{baseclass}{vpa@type}{%
    {habil}{0},{diss}{1},{diplom}{2},%
    {sta}{3},{beleg}{4},{other}{5}%
  }{#1}%
  \@vpa@studentfalse%
  \ifx\FamilyKeyState\FamilyKeyStateProcessed%
    \ifcase\vpa@type\relax% habil
    \or% diss
    \or% diplom
      \@vpa@studenttrue%
    \or% sta
      \@vpa@studenttrue%
    \or% beleg
      \@vpa@studenttrue%
    \or% other
    \fi%
  \fi%
}
\newcommand*\vpa@pass@options{}
\DeclareOption*{\eappto\vpa@pass@options{,\expandonce\CurrentOption}}%
\FamilyProcessOptions{vpa}\relax
\ifx\vpa@baseclass\relax%
  \ifnum\vpa@type>\tw@\relax%
    \FamilyOptions{vpa}{baseclass=report}%
  \else%
    \FamilyOptions{vpa}{baseclass=book}%
  \fi%
\fi%
\PassOptionsToPackage{
  DIV=12,
  BCOR=5mm,
  paper=a4,
  headinclude=true,
  footinclude=false,
}{typearea}
\PassOptionsToClass{%
  cdfont=false,
  cd=false,
  cdtitle=true,
  cdgeometry=false,
  abstract=section,
  abstract=nofil,
  abstract=tocmultiple,
% TODO bugfix paper=a4
  fontsize=11pt,
  twoside,
  open=right,
  numbers=noenddot,
  captions=tableheading,
  listof=totoc,
  % optionales Argument der Gliederungsbefehle erweitert (s. scrguide.pdf)
  headings=optiontoheadandtoc,
  \vpa@pass@options
}{\vpa@baseclass}
\LoadClass{\vpa@baseclass}
\newcommand*\vpa@@keywords{}
\newcommand*\vpa@@keywords@english{}
\newcommand*\vpa@@keywords@german{}
\newcaptionname{ngerman,german}{\vpa@keywords}{\vpa@@keywords@german}
\newcaptionname{english}{\vpa@keywords}{\vpa@@keywords@english}
\newcommand*\addkeywords[2][]{%
  \begingroup%
    \def\@tempa##1##2{%
      \def\@tempb{##2}%
      \trim@spaces@in\@tempb%
      \expandafter\ifinlist\expandafter{\@tempb}{##1}{}{%
        \listxadd##1{\expandonce\@tempb}%
      }%
    }%
    \forcsvlist{\@tempa{\vpa@@keywords@german}}{#1}%
    \forcsvlist{\@tempa{\vpa@@keywords@english}}{#2}%
  \endgroup%
}
% TODO warum funktioniert datetime2 nicht richtig?
\NewDocumentCommand\vpa@localization@german{s}{%
  \IfBooleanTF{#1}{\providecaptionname}{\defcaptionname}%
  {german,ngerman,austrian,naustrian,swissgerman,nswissgerman}%
}
\NewDocumentCommand\vpa@localization@english{s}{%
  \IfBooleanTF{#1}{\providecaptionname}{\defcaptionname}%
  {american,australian,british,canadian,english,newzealand,UKenglish,USenglish}%
}
\AfterClass!{\vpa@baseclass}{%
  \addtokomafont{disposition}{\cdfontsn}
  \faculty{%
    Fakult\"at Verkehrswissenschaften %
    \quotedblbase Friedrich List\textquotedblleft%
  }
  \institute{Institut f\"ur Verkehrstelematik}
  \chair{Professur f\"ur Verkehrsprozessautomatisierung}
  \newcommand*\vpa@professor{Prof. Dr. Meng Wang}
  \expandafter\professor\expandafter{\vpa@professor}%
  % TODO für v2.07 überarbeiten
  \vpa@localization@german\professorname{%
    \ifx\vpa@professor\@professor%
      Verantwortliche Hochschullehrerin%
    \else%
      \tud@ifin@and{\@professor}{%
        \ifx\professorothername\@empty%
          Betreuende Hochschullehrer%
        \else%
          Erster betreuender Hochschullehrer%
        \fi%
      }{Betreuender Hochschullehrer}%
    \fi%
  }%
  \vpa@localization@english\professorname{%
    \ifx\vpa@professor\@professor%
      Responsible Professor%
    \else%
      \tud@ifin@and{\@professor}{%
        \ifx\professorothername\@empty%
          Supervising Professors%
        \else%
          First Supervising Professor%
        \fi%
      }{Supervising Professor}%
    \fi%
  }%
  % TODO für v2.07 überarbeiten
  \vpa@localization@german\supervisorname{%
    \tud@ifin@and{\@supervisor}{%
      \ifx\supervisorothername\@empty%
        Betreuende%
      \else%
        Erstbetreuer%
      \fi%
    }{Betreuer}%
  }%
  \vpa@localization@english\supervisorname{%
    \tud@ifin@and{\@supervisor}{%
      \ifx\supervisorothername\@empty%
        Supervisors%
      \else%
       First Supervisor%
      \fi%
    }{Supervisor}%
  }%
  % TODO für v2.07 überarbeiten
  \if@vpa@student%
    \vpa@localization@german\abstractname{Autorenreferat}%
    \vpa@localization@english\abstractname{Decleration of academic integrity}%
  \fi%
  \place{Dresden}%
  % TODO map to tudscr types and corresponding localization
  \ifcase\vpa@type\relax% habil
    \graduation{Dr.-Ing. habil.}%
    \thesis{Habilitation}%
  \or% diss
    \graduation[Dr.-Ing.]{Doktor-Ingenieur}%
    \thesis{Dissertation}%
  \or% diplom
    \graduation[Dipl.-Ing.]{Diplom-Ingenieur}%
    \thesis{diploma}%
  \or% sta
    \thesis{student}%
  \or% beleg
    \thesis{seminar}%
  \or% other
  \fi%
  \InputIfFileExists{metadata}{}{}%
}
\PassOptionsToPackage{%
  plainpages=false,
  hypertexnames=false,
  breaklinks=true,
%  pdfpagelabels=true,
%  pdfa=true
}{hyperref}
\newcommand*\vpa@pdf@info{}
\AfterPackage{hyperref}{%
  \renewcommand*\vpa@pdf@info{%
    \begingroup%
      \def\and{, }%
      \let\thanks\@gobble%
      \let\footnote\@gobble%
      \let\vpa@@keywords\@empty%
      \def\@tempa##1{%
        \ifx\vpa@@keywords\@empty\else%
          \appto\vpa@@keywords{,}%
        \fi%
        \appto\vpa@@keywords{##1}%
      }%
      \forlistloop\@tempa{\vpa@@keywords@english}%
      \forlistloop\@tempa{\vpa@@keywords@german}%
      \def\@tempa##1{\cslet{##1}\@gobble}%
      \expandafter\forcsvlist\expandafter\@tempa\expandafter{%
        \tud@split@author@list%
      }%
      \hypersetup{%
        pdfauthor   = {\@author},%
        pdftitle    = {\@title},%
        % TODO
%        pdfsubject  = {\@subtitle},%       \@subject vs. \@@thesis
        pdfkeywords = {\vpa@@keywords},%
      }%
    \endgroup%
  }%
}
\newif\if@mathsub@upright
\@tempswatrue
\ifundef{\enableuprightsubscripts}{}{\@tempswafalse}
\ifundef{\disableuprightsubscripts}{}{\@tempswafalse}
\ifundef{\swapuprightsubscripts}{}{\@tempswafalse}
\if@tempswa
  % TODO option
  % TODO catcodes in marcos für true/false
  \AtBeginDocument{\mathcode`\_="8000}%
  \newcommand*\mathsub@set[1]{%
    \if@mathsub@upright%
      \sb{\mathrm{#1}}%
    \else%
      \sb{\mathit{#1}}%
    \fi%
  }%
  \newcommand*\mathsub@swap{}
  \def\mathsub@swap|#1|{%
    \begingroup%
      \swapuprightsubscripts%
      \mathsub@set{#1}%
    \endgroup%
  }%
  \begingroup%
    \catcode`\_=13%
    \gdef_{\@ifnextchar|\mathsub@swap\mathsub@set}%
  \endgroup%
  \newcommand*\enableuprightsubscripts{\@mathsub@uprighttrue}%
  \newcommand*\disableuprightsubscripts{\@mathsub@uprightfalse}%
  \newcommand*\swapuprightsubscripts{%
    \if@mathsub@upright%
      \@mathsub@uprightfalse%
    \else%
      \@mathsub@uprighttrue%
    \fi%
  }%
\fi
\ifcsundef{tud@v@2.07}{\@tempswatrue}{\@tempswafalse}
\if@tempswa
  % TODO patch raus, sobald das in tudscr gefixt ist
  \xapptocmd\tud@maketitle{%
    \begingroup%
      \edef\@tempa{%
        \endgroup%
        \ifcase\tud@cd@title@num\relax\else% !false
          \if@titlepage%
            \unexpanded{\let\tud@currentgeometry\@empty}%
            \noexpand\tud@loadgeometry{\tud@currentgeometry}%
          \fi%
        \fi%
      }%
    \@tempa%
  }{}{\tud@patch@wrn{tud@maketitle}}%
\fi
% TODO new data type for @standard and \DeclareLabelalphaTemplate[standard]{}
% https://tex.stackexchange.com/questions/175776
\AfterPackage{biblatex}{%
  \RequirePackage{url}
  \renewcommand*{\labelalphaothers}{}
  \renewcommand*{\mkbibnamefamily}[1]{\textsc{#1}}
  \renewcommand*{\labelnamepunct}{\addcolon\addspace}
  \DeclareLabelalphaTemplate{%
    \labelelement{%
      \field[final]{shorthand}
      \field{label}
      \field[strwidth=3, names=1]{labelname}
      \field[strwidth=3]{editor}
      \field[strwidth=3]{number}
    }
    \labelelement{%
      \field[strwidth=2,strside=right]{year}
    }
  }
  \DeclareFieldFormat[patent]{note}{\bibstring{anmeldetag} #1}
  \DeclareFieldFormat[patent]{date}{(#1)}
  % Bibliography strings
  \NewBibliographyString{patent}
  \NewBibliographyString{norm}
  \NewBibliographyString{anmeldetag}
  \DefineBibliographyStrings{german}{
    norm       = Norm,
    patent     = Schutzrecht,
    anmeldetag = Pr. (Anmeldetag),
  }
  \DefineBibliographyStrings{ngerman}{
    norm       = Norm,
    patent     = Schutzrecht,
    anmeldetag = Pr. (Anmeldetag),
  }
  \DefineBibliographyStrings{english}{
    norm       = Standard,
    patent     = Patent,
  }
  \DeclareBibliographyDriver{online}{%
    \usebibmacro{bibindex}%
    \usebibmacro{begentry}%
    \ifnameundef{author}
      {\iflistundef{organization}
        {\BibliographyWarning{No author nor organisation on Online entry}}
        {\printlist{organization}}}
      {\usebibmacro{author/editor+others/translator+others}}
    \setunit{\labelnamepunct}\newblock
    \usebibmacro{title}%
    \newunit
    \printlist{language}%
    \newunit\newblock
    \usebibmacro{byauthor}%
    \newunit\newblock
    \usebibmacro{byeditor+others}%
    \newunit\newblock
    \printfield{version}%
    \newunit
    \printfield{note}%
    \newunit\newblock
    \ifnameundef{author}
      {}
      {\printlist{organization}}
    \newunit\newblock
    \usebibmacro{date}%
    \newunit\newblock
    \iftoggle{bbx:eprint}
      {\usebibmacro{eprint}}
      {}%
    \newunit\newblock
    \usebibmacro{url+urldate}%
    \newunit\newblock
    \usebibmacro{addendum+pubstate}%
    \setunit{\bibpagerefpunct}\newblock
    \usebibmacro{pageref}%
    \usebibmacro{finentry}%
  }
  \DeclareBibliographyDriver{patent}{%
    \usebibmacro{bibindex}%
    \usebibmacro{begentry}%
    \bibstring{patent}
    \newblock
    \printfield{number}%
    \newblock\setunit*{\addspace}
    \usebibmacro{date}
    \newunit\newblock
    \printfield{note}%
    \usebibmacro{author}
    \setunit{\labelnamepunct}\newblock
    \usebibmacro{title}%
    \newunit
    \printlist{language}%
    \setunit{\bibpagerefpunct}\newblock
    \usebibmacro{pageref}
    \newunit\newblock
    \usebibmacro{related}
    \usebibmacro{finentry}
  }
  \DeclareBibliographyDriver{misc}{%
    \usebibmacro{bibindex}%
    \usebibmacro{begentry}%
    \bibstring{norm}
    \printfield{type}%
    \newblock\setunit*{\addspace}
    \printfield{number}%
    \newblock\setunit*{\addspace}
    \usebibmacro{date}
    \newunit\newblock
    \printfield{note}%
    \usebibmacro{author}
    \setunit{\labelnamepunct}\newblock
    \usebibmacro{title}%
    \newunit
    \printlist{language}%
    \setunit{\bibpagerefpunct}\newblock
    \usebibmacro{pageref}
    \newunit\newblock
    \usebibmacro{related}
    \usebibmacro{finentry}
  }
}
\AtBeginDocument{%
  \InputIfFileExists{hyphenations}{}{}%
  % schräggestellter bruch
  \providecommand*\nfrac[2]{%
    \leavevmode%
    \kern.1em\raise.5ex\hbox{\scriptsize #1}%
    \kern-.1em/\kern-.15em%
    \lower.25ex\hbox{\scriptsize #2}%
  }%
  \renewcommand*\labelitemi{\normalfont\bfseries\textendash}%
  \renewcommand*\labelitemii{\textbullet}%
}
\newcommand*\vpa@temp{}
\newcommand*\vpa@get@keywords{%
  \begingroup%
    \ifx\vpa@@keywords@german\@empty%
      \let\vpa@@keywords@german\vpa@@keywords@english%
    \fi%
    \let\vpa@@keywords\@empty%
    \def\vpa@temp##1{%
      \ifx\vpa@@keywords\@empty\else%
        \appto\vpa@@keywords{,\space}%
      \fi%
      \appto\vpa@@keywords{##1}%
    }%
    \expandafter\forlistloop\expandafter\vpa@temp\expandafter{\vpa@keywords}%
    \edef\vpa@@keywords{%
      \endgroup%
      \unexpanded{\par\bigskip\noindent}%
      \expandonce\vpa@@keywords%
    }%
  \vpa@@keywords%
}
\pretocmd\tud@fm@body@end{%
  \Ifstr{#1}{abstract}{\vpa@get@keywords}{}%
}{}{\tud@patch@wrn{tud@fm@body@end}}%
\AfterClass!{\vpa@baseclass}{%
  \vpa@localization@german*\pagename{Seite}%
  \vpa@localization@german*\pagesname{Seiten}%
  \vpa@localization@german*\figurename{Abbildung}%
  \vpa@localization@german*\figuresname{Abbildungen}%
  \vpa@localization@german*\tablename{Tabelle}%
  \vpa@localization@german*\tablesname{Tabellen}%
  \vpa@localization@german*\refcitename{Quelle}%
  \vpa@localization@german*\refcitesname{Quellen}%
  \vpa@localization@german*\appendixname{Anhang}%
  \vpa@localization@german*\bibrecordname{Bibliografischer Nachweis}%
  \vpa@localization@german*\listappendixname{Verzeichnis der Anh\"ange}%
  \vpa@localization@english*\pagename{Page}%
  \vpa@localization@english*\pagesname{Pages}%
  \vpa@localization@english*\figurename{Figure}%
  \vpa@localization@english*\figuresname{Figures}%
  \vpa@localization@english*\tablename{Table}%
  \vpa@localization@english*\tablesname{Tables}%
  \vpa@localization@english*\refcitename{Reference}%
  \vpa@localization@english*\refcitesname{References}%
  \vpa@localization@english*\appendixname{Appendix}%
  \vpa@localization@english*\bibrecordname{Bibliographical Record}%
  \vpa@localization@english*\listappendixname{List of Annexes}%
}
\newcommand*\vpa@bib@reference{%
  \begingroup%
    \iftotalpages
      \ifundef\vpa@main@pages{%
        \def\vpa@temp{%
          \vpa@numerus*{\totalpages-1\if@vpa@student-1\fi}{page}%
          \vpa@numerus{\totalfigures}{figure}%
          \vpa@numerus{\totaltables}{table}%
          \vpa@numerus{\vpa@cites}{refcite}%
        }%
      }{%
        \def\vpa@temp{%
          \vpa@numerus*{\vpa@main@pages}{page}%
          \vpa@numerus{\vpa@main@figures}{figure}%
          \vpa@numerus{\vpa@main@tables}{table}%
          \vpa@numerus{\vpa@cites}{refcite}
          \newline\expanded{\noexpand\MakeUppercase\appendixname}:\space%
          \vpa@numerus*{\totalpages-1\if@vpa@student-1\fi-\vpa@main@pages}{page}%
          \vpa@numerus{\totalfigures-\vpa@main@figures}{figure}%
          \vpa@numerus{\totaltables-\vpa@main@tables}{table}%
        }%
      }%
    \else%
      \let\vpa@temp\relax%
    \fi%
    \addsec*{\bibrecordname}%
    \begin{trivlist}
      \item \getfield{author}\\
            \getfield{title}%
      \item \getfield{thesis}, Technische Universit\"at Dresden\\
            \getfield{faculty}%
      \ifx\vpa@temp\relax\else%
        \item \vpa@temp%
      \fi%
    \end{trivlist}
  \endgroup%
}
\newcount\vpa@cnt
% whether the numerical expression evaluates to '1' appropriate locale is used
\newcommand*\vpa@numerus@delim{}
\NewDocumentCommand\vpa@numerus{smm}{%
  \IfBooleanT{#1}{\let\vpa@numerus@delim\@empty}%
  \vpa@cnt=\numexpr#2\relax%
  \ifnum\vpa@cnt>\z@
    \vpa@numerus@delim\the\vpa@cnt\nobreakspace%
    \expanded{%
      \noexpand\MakeUppercase%
      \csname#3\ifnum\vpa@cnt=\@ne\else s\fi name\endcsname
    }%
    \def\vpa@numerus@delim{,\space}%
  \fi
}
\newcommand*\vpa@footnote{}
\newcommand*\vpa@currentpagestyle{}
\newcommand*\vpafrontmatter[2][]{%
  \vpa@pdf@info%
  \renewcommand*\addkeywords[2][]{%
    \ClassError{tudscr-vpa}{Command \string\addkeywords\space too late}{%
      The command \string\addkeywords\space can only be used\MessageBreak%
      before \string\vpafrontmatter.%
    }%
  }%
  \let\vpa@footnote\thefootnote%
  \let\vpa@currentpagestyle\currentpagestyle%
  \if@vpa@student\else%
    \IfFileExists{declaration}{%
      \uppertitleback{\input{declaration}}%
    }{}%
    \lowertitleback{\vpa@bib@reference}%
  \fi%
  \maketitle[{cdgeometry=true,#1}]%
  \renewcommand*\thefootnote{\footnotesize\textrm\Roman{footnote}}%
  \if@vpa@student%
    \begingroup%
      \pagestyle{empty}%
      \KOMAoptions{titlepage=false}%
      \TUDoptions{abstract=notoc}%
      \input{declaration}%
      \vfill%
      \vpa@bib@reference%
      \cleardoublepage%
    \endgroup%
  \fi%
  \pagestyle{plain}%
  \csname frontmatter\endcsname%
  \setcounter{page}{1}%
  \pagenumbering{Roman}%
  \InputIfFileExists{preface}{}{}%
  \if@vpa@student%
    \addchap*{Thesen}%
    \tudbookmark[0]{Thesen}{theses}%
    \input{assumptions}%
  \fi%
  \tableofcontents%
  \cleardoublepage%
  #2%
  \let\thefootnote\vpa@footnote%
  \setcounter{footnote}{0}%
  \cleardoublepage%
  \csname mainmatter\endcsname%
  \setcounter{page}{1}%
  \pagenumbering{arabic}%
  \expandafter\pagestyle\expandafter{\vpa@currentpagestyle}%
  \renewcommand*\vpafrontmatter[2][]{%
    \ClassWarning{tudscr-vpa}{\string\vpafrontmatter\space was already used}%
  }%
}
\newcommand*\vpa@backmatter[1]{%
  \ClassWarning{tudscr-vpa}{\string\vpabackmatter\space was already used}%
}
\newcommand*\vpabackmatter[1]{%
  \csname backmatter\endcsname
  #1%
  \if@vpa@student\confirmation\fi%
  \let\vpabackmatter\vpa@backmatter%
}
\AtEndDocument{%
  \ifx\vpabackmatter\vpa@backmatter\else%
    \vpabackmatter{}%
  \fi%
}
\RequirePackage{totalcount}
\DeclareTotalCounter{figure}
\DeclareTotalCounter{table}
\DeclareTotalCounter{page}
\newcommand*\vpa@appendix{%
  \clearpage%
  \if@filesw%
    \immediate\write\@mainaux{%
      \string\gdef\string\vpa@main@pages{\the\numexpr\value{page}-1\relax}^^J%
      \string\gdef\string\vpa@main@figures{\the\value{totalcount@figure}}^^J%
      \string\gdef\string\vpa@main@tables{\the\value{totalcount@table}}%
    }%
  \fi%
  \cleardoubleemptypage%
  \begingroup%
    \renewcommand*\partpagestyle{empty}%
    \addpart{\appendixname}%
  \endgroup%
  \renewcommand*\ext@toc{atoc}%
  \listoftoc[\listappendixname]{atoc}%
  \cleardoubleemptypage%
}
\ifundef\appendixmore{\def\appendixmore{}}{}
\appto\appendixmore{\vpa@appendix}
\newcommand*\vpa@cites{0}
\AfterPackage{biblatex}{%
  \newcounter{vpa@cites}
  \AtDataInput{\stepcounter{vpa@cites}}
  \AtEndDocument{%
    \if@filesw%
      \immediate\write\@mainaux{%
        \string\gdef\string\vpa@cites{\the\value{vpa@cites}}%
      }%
    \fi%
  }%
}
\endinput
